
// mfc_simpleView.cpp : implementation of the Cmfc_simpleView class
//

#include "stdafx.h"
// SHARED_HANDLERS can be defined in an ATL project implementing preview, thumbnail
// and search filter handlers and allows sharing of document code with that project.
#ifndef SHARED_HANDLERS
#include "mfc_simple.h"
#endif

#include "mfc_simpleDoc.h"
#include "mfc_simpleView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// Cmfc_simpleView

IMPLEMENT_DYNCREATE(Cmfc_simpleView, CView)

BEGIN_MESSAGE_MAP(Cmfc_simpleView, CView)
	// Standard printing commands
	ON_COMMAND(ID_FILE_SAVE_AS, &Cmfc_simpleView::OnFileSaveAs)
	ON_COMMAND(ID_FILE_PRINT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, &CView::OnFilePrintPreview)
END_MESSAGE_MAP()

// Cmfc_simpleView construction/destruction

Cmfc_simpleView::Cmfc_simpleView()
{
	// TODO: add construction code here

}

Cmfc_simpleView::~Cmfc_simpleView()
{
}

void Cmfc_simpleView::OnFileSaveAs()
{
	TCHAR filters[] = {_T("pdf (*.pdf)|*.pdf")};
	CFileDialog fileDlg(false, _T("pdf"), _T("*.pdf"), 0, filters);

	if (fileDlg.DoModal() == IDOK)
	{
		HPS::Hardcopy::File::ExportOptionsKit options;
		options.SetSize(8.5, 11);
		options.SetResolution(100);
		CString fn = fileDlg.GetPathName();
		HPS::UTF8 utf8(fn);
		HPS::Hardcopy::File::Export(utf8, HPS::Hardcopy::File::Driver::PDF, _canvas.GetWindowKey(), options);
	}
}

BOOL Cmfc_simpleView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

void Cmfc_simpleView::OnInitialUpdate()
{
	// Only perform HPS::Canvas initialization once since we're reusing the same MFC Window each time.
	if (_canvas.Type() == HPS::Type::None)
	{
		// Setup to use the DX9 driver
		HPS::ApplicationWindowOptionsKit		windowOpts;
		windowOpts.SetDriver(HPS::Window::Driver::DirectX9);

		// Create our Sprockets Canvas with the specified driver
		_canvas = HPS::Factory::CreateCanvas(reinterpret_cast<HPS::WindowHandle>(m_hWnd), "mfc_sandbox_canvas", windowOpts);

		// Create a new Sprockets View and attach it to our Sprockets.Canvas
		HPS::View view = HPS::Factory::CreateView("mfc_sandbox_view");
		_canvas.AttachViewAsLayout(view);

		// Setup camera
		view.AttachModel(GetDocument()->_model);
		view.FitWorld();

		// Set default operators.  Orbit is on top and will be replaced when the operator is changed
		if (view.GetOperatorControl().GetCount() == 0)
		{
			view.GetOperatorControl()
				.Push(new HPS::MouseWheelOperator(), HPS::Operator::Priority::Low)
				.Push(new HPS::ZoomOperator(HPS::MouseButtons::ButtonMiddle()))
				.Push(new HPS::PanOperator(HPS::MouseButtons::ButtonRight()))
				.Push(new HPS::OrbitOperator(HPS::MouseButtons::ButtonLeft()));
		}

		// Set default colors
		_canvas.GetWindowKey().GetMaterialMappingControl()
			.SetWindowColor(HPS::RGBAColor(0, 0, 160.0f / 255.0f))
			.SetWindowContrastColor(HPS::RGBAColor(1, 1, 1))
			.SetEdgeColor(HPS::RGBAColor(0.2f, 0.4f, 0.2f));

		// Set our background gradient
		_canvas.GetWindowKey().GetSubwindowControl()
			.SetBackground(HPS::Subwindow::Background::GradientTopToBottom);

		// Enable display lists for performance
		_canvas.GetWindowKey().GetPerformanceControl()
			.SetDisplayLists(HPS::Performance::DisplayLists::SegmentDisplayLists);

		// Enable 10px extent culling
		_canvas.GetWindowKey().GetCullingControl().SetExtent(10);

		// Set selection level
		_canvas.GetWindowKey().GetSelectionOptionsControl().SetLevel(HPS::Selection::Level::Segment);
	}
	
	// Invoke update to ensure views/layouts attached
	_canvas.Update();
}

// Cmfc_simpleView drawing
//! [mfc_print]
void Cmfc_simpleView::OnPrint(CDC* pDC, CPrintInfo* pInfo)
{
	Hardcopy::GDI::ExportOptionsKit options;

	if (pInfo->m_bPreview == TRUE)
		options.SetResolution(50);

	Hardcopy::GDI::Export((long) pDC->m_hDC,
                          (long) pDC->m_hAttribDC,
                          _canvas.GetWindowKey(),
						  options);

}
//! [mfc_print]

void Cmfc_simpleView::OnDraw(CDC* /*pDC*/)
{
	Cmfc_simpleDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	HPS::UpdateNotifier notifier = _canvas.UpdateWithNotifier(HPS::Window::UpdateControl::RefreshUpdate);
	notifier.Wait();
	// TODO: add draw code for native data here
}


// Cmfc_simpleView printing

BOOL Cmfc_simpleView::OnPreparePrinting(CPrintInfo* pInfo)
{
	pInfo->m_nNumPreviewPages = 1;
	// default preparation
	return DoPreparePrinting(pInfo);
}

void Cmfc_simpleView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void Cmfc_simpleView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


// Cmfc_simpleView diagnostics

#ifdef _DEBUG
void Cmfc_simpleView::AssertValid() const
{
	CView::AssertValid();
}

void Cmfc_simpleView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

Cmfc_simpleDoc* Cmfc_simpleView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(Cmfc_simpleDoc)));
	return (Cmfc_simpleDoc*)m_pDocument;
}
#endif //_DEBUG


// Cmfc_simpleView message handlers
