#include "SamplesOSX.h"
#include <algorithm>
#include <string>
#include <unistd.h>
#include <stdio.h>

using namespace HPS;

Sample::Sample()
{
	testToRun = new Test();
}

Sample::~Sample()
{
	delete testToRun;
}

void Sample::RunTest(Test* test)
{
    if (canvas.Type() == HPS::Type::None)
    {
        canvas = Factory::CreateCanvas(wk);
        view = Factory::CreateView();
        canvas.AttachViewAsLayout(view);
    }
    
    if (model.Type() != HPS::Type::None)
    {
        canvas.GetWindowKey().GetPostProcessEffectsControl().SetBloom(false).SetSilhouetteEdges(false);
        view.DetachModel();
        model.Delete();
    }
    
    model = Factory::CreateModel();
	view.AttachModel(model);
	SegmentKey modelKey = model.GetSegmentKey();
    canvas.GetWindowKey().GetSubwindowControl().SetBackground(HPS::Subwindow::Background::GradientTopToBottom);
    canvas.GetWindowKey().GetMaterialMappingControl().SetWindowColor(HPS::RGBAColor(0, 0, 1, 0));
    
	modelKey.InsertDistantLight(Vector(1, 1, 0));

    
	switch(test->testID)
	{
        case 0:
            Lightweight_Subwindow(wk, canvas, view, model, modelKey);
            break;
        case 1:
            Bloom_Silhouette(wk, canvas, view, model, modelKey);
            break;
        case 2:
            Database_Search(wk, canvas, view, model, modelKey);
            break;
        case 3:
            Hidden_Line(wk, canvas, view, model, modelKey);
            break;
        case 4:
            Image_Background(wk, canvas, view, model, modelKey);
            break;
        case 5:
            Image_Io(wk, canvas, view, model, modelKey);
            break;
        case 6:
            Image_Output(wk, canvas, view, model, modelKey);
            break;
        case 7:
            Mask_Transforms(wk, canvas, view, model, modelKey);
            break;
        case 8:
            Meshes(wk, canvas, view, model, modelKey);
            break;
        case 9:
            Quickmoves(wk, canvas, view, model, modelKey);
            break;
        case 10:
            Render_Modes(wk, canvas, view, model, modelKey);
            break;
        case 11:
            Segment_Attributes(wk, canvas, view, model, modelKey);
            break;
        case 12:
            Sub_Shell_Attributes(wk, canvas, view, model, modelKey);
            break;
        case 13:
            Spotlight(wk, canvas, view, model, modelKey);
            break;
        case 14:
            Textures(wk, canvas, view, model, modelKey);
            break;
        case 15:
            Exchange_Publish(wk, canvas, view, model, modelKey);
            break;
        case 16:
            Hidden_Line_Advanced(wk, canvas, view, model, modelKey);
            break;
        case 17:
            Shell_Attributes(wk, canvas, view, model, modelKey);
            break;
        case 18:
            Priority(wk, canvas, view, model, modelKey);
            break;
        case 19:
            Interpolation(wk, canvas, view, model, modelKey);
            break;
        case 20:
            Exchange_PMI_Highlighting(wk, canvas, view, model, modelKey);
            break;
        case 21:
            Exchange_Component_Highlighting(wk, canvas, view, model, modelKey);
            break;
        case 22:
            Custom_Event(wk, canvas, view, model, modelKey);
            break;
        case 23:
            Conditional_Styles_And_Includes(wk, canvas, view, model, modelKey);
            break;
        case 24:
            Hardcopy_Export(wk, canvas, view, model, modelKey);
            break;
	}
    
    view.FitWorld();
    canvas.UpdateWithNotifier(HPS::Window::UpdateControl::ExhaustiveUpdate);
}
