using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Hardcopy_Export(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            /* Sample: Hardcopy_Export
	         * This sample will show you how to:
	         * - Import an HSF file
	         * - Retrieve the size of the window at runtime
	         * - Export a 2D-PDF and PostScript file
	         * - Check whether the Hardcopy Export completed successfully
	         * 
	         * Read more about the topics covered in this sample in our Programming Guide 
	         * - section x.x:	 */

	        /* This next block of code shows you how to load an HSF file
	         * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
	         * 2. Use the Stream::File::Import method to load the desired file, with the options you specified
	         * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
	         * 4. Use a try-catch block to capture any Stream IO exceptions */
	        Stream.ImportNotifier	notifier = new Stream.ImportNotifier();
	        SegmentKey geometry = modelKey.Subsegment();

	        try
	        {
		        Stream.ImportOptionsKit importOptionsKit = new Stream.ImportOptionsKit();
		        importOptionsKit.SetSegment(geometry);
		        string filename = INPUT_FOLDER + "/conrod.hsf";
		
		        notifier = Stream.File.Import(filename, importOptionsKit);
		        notifier.Wait();
	        }
	        catch (IOException)
	        {
		        /* catch Stream I/O Errors here */
	        }

	        view.FitWorld();
	        view.Update();

	        /* This block of code shows you how to export a 2D-PDF and a Postscript file representing the scene
	         * 1. Use a Hardcopy::File::ExportOptionKit object to set the options needed for the export 
	         * 2. To obtain the size of a window, get its WindowInfoControl object and use its ShowPhysicalSize method
	         * 3. Setting WYSIWYG on the export kit matches the model on screen to the hardcopy output exactly
	         * 4. The Hardcopy::File:Export method is where the format to export to is specified
	         * 5. Hardcopy::File::Export returns a Hardcopy::IOResult object which can be checked to see if the
	         *		export operation completed successfully. */
	        IOResult PDFResult = IOResult.Failure, postScriptResult = IOResult.Failure;
	        try
	        {
		        string file_out = OUTPUT_FOLDER + "/hardcopy_export.pdf";
		        Hardcopy.File.ExportOptionsKit exportOptionsKit = new Hardcopy.File.ExportOptionsKit();
		        float width, height;

		        wk.GetWindowInfoControl().ShowPhysicalSize(out width, out height);
		        exportOptionsKit.SetWYSIWYG(true);
		        exportOptionsKit.SetSize(width, height, Hardcopy.SizeUnits.Centimeters);
                exportOptionsKit.SetResolution(100, Hardcopy.ResolutionUnits.DPCM);

		        PDFResult = Hardcopy.File.Export(file_out, Hardcopy.File.Driver.PDF, wk, exportOptionsKit);
		        file_out = OUTPUT_FOLDER + "/hardcopy_export.ps";
		        postScriptResult = Hardcopy.File.Export(file_out, Hardcopy.File.Driver.Postscript, wk, exportOptionsKit);
	        }
	        catch (IOException)
	        {
		        //handle Hardcopy exception
	        }

	        /* This block of code checks the result of the two export operations completed above
	         * and prints out a message on the screen informing the user whether they completed
	         * successfully or not*/
	        SegmentKey description = modelKey.Subsegment();
	        description.GetDrawingAttributeControl().SetDepthRange(0.0f, 0.1f);
	        description.GetVisibilityControl().SetEverything(false).SetText(true);
	        description.GetTextAttributeControl().SetAlignment(Text.Alignment.Center, Text.ReferenceFrame.WorldAligned, Text.Justification.CenterJustified);
	        description.GetCameraControl().SetPosition(new Point(0, 0, -5)).SetTarget(new Point(0, 0, 0)).SetUpVector(new Vector(0, 1, 0)).
		        SetField(2, 2).SetProjection(Camera.Projection.Stretched);

	        string resultDescription;
	        if(PDFResult == IOResult.Success)
		        resultDescription = "PDF Export completed successfully";
	        else
		        resultDescription = "PDF Export FAILED";

	        if(postScriptResult == IOResult.Success)
		        resultDescription = resultDescription + "\nPostScript Export completed successfully";
	        else
		        resultDescription = resultDescription + "\nPostScript Export FAILED";

	        resultDescription = resultDescription + "\nYou can find the files in this folder:\n" + OUTPUT_FOLDER;

	        description.InsertText(new Point(0, 0.80f, 0), resultDescription);
        }
    }
}
