using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Highlighting(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            StandAloneWindowKey myWindowKey = (StandAloneWindowKey)wk;
            /* Sample: Highlight styles
            * This sample will show you how to:
            * - Load an HSF file
            * - Load an image and use it to create a texture
            * - Create an opaque style
            * - Create a transparent style
            * - Create a textured style
            *
            * Read more about the topics covered in this sample in our Programming Guide
            * - section 6.4:	Highlighting
            * - section 4.3:    Defining and using styles
            */


            ImageKit imageKit = new ImageKit();

            try
            {
                /* This next block of code shows you how to load an HSF file
                * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
                * 2. Use the Stream::File::Import method to load the desired file with the options you specified
                * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding
                * 4. Use a try-catch block to capture any Stream IO exceptions */
                Stream.ImportOptionsKit importOptionsKit = new Stream.ImportOptionsKit();
                importOptionsKit.SetSegment(modelKey);
                String filename = INPUT_FOLDER + "/bnc.hsf";

                Stream.ImportNotifier notifier = Stream.File.Import(filename, importOptionsKit);
                notifier.Wait();

                /* This block of code shows you how to load an iamge from a file
                * 1. Create an ImportOptionsKit and set the image format
                * 2. Use the Image::File::Import method to load the desired file with the options you specified
                * 3. Use a try-catch block to capture any IO exceptions */
                filename = INPUT_FOLDER + "/pattern.jpg";
                Image.ImportOptionsKit iok = new Image.ImportOptionsKit();
                iok.SetFormat(Image.Format.Jpeg);
                imageKit = Image.File.Import(filename, iok);
            }
            catch (IOException)
            {
                // handle IO errors here
            }

            // create a portfolio to store styles and the texture
            PortfolioKey myPortfolio = Database.CreatePortfolio();

            // create an ImageDefinition from the image loaded earlier. an image definition
            // is required to create a TextureDefinition
            ImageDefinition imageDefinition = myPortfolio.DefineImage("my_image", imageKit);
            TextureOptionsKit textureOptionsKit = new TextureOptionsKit();

            // see the table below about parameterization sources
            textureOptionsKit.SetParameterizationSource(Material.Texture.Parameterization.UV);

            // makes the image into a texture
            myPortfolio.DefineTexture("my_texture", imageDefinition, textureOptionsKit);

            modelKey.GetModellingMatrixControl().Rotate(-90, 30, 0).Scale(0.6f, 0.6f, 0.6f);


            // create the opaque red style
            SegmentKey redStyleSegment = Database.CreateRootSegment();
            NamedStyleDefinition red_style = myPortfolio.DefineNamedStyle("red_style", redStyleSegment);
            redStyleSegment.GetMaterialMappingControl().SetFaceColor(new RGBAColor(1.0f, 0, 0));

            // create the transparent style. it is a 1/2 transparent red style
            SegmentKey transparentStyleSegment = Database.CreateRootSegment();
            NamedStyleDefinition transparent_style = myPortfolio.DefineNamedStyle("transparent_style", transparentStyleSegment);
            transparentStyleSegment.GetMaterialMappingControl().SetFaceColor(new RGBAColor(1.0f, 0, 0, 0.5f));

            // create the textured style based on the image we loaded earlier
            SegmentKey texturedStyleKey = Database.CreateRootSegment();
            NamedStyleDefinition textured_style = myPortfolio.DefineNamedStyle("textured_style", texturedStyleKey);
            textured_style.GetSource().GetMaterialMappingControl().SetFaceTexture("my_texture");

            // set up the highlight options kits
            HighlightOptionsKit hok1 = new HighlightOptionsKit("red_style");
            HighlightOptionsKit hok2 = new HighlightOptionsKit("transparent_style");
            HighlightOptionsKit hok3 = new HighlightOptionsKit("textured_style");


            // make the portfolio active
            modelKey.GetPortfolioControl().Push(myPortfolio);

            HighlightOperator highlightOperator
                = new HighlightOperator(MouseButtons.ButtonLeft());
            view.GetOperatorControl().Push(highlightOperator); // makes 'highlightOperator' active

            view.FitWorld();

            TextKit textKit = new TextKit();
            textKit.SetText("Unstyled object\n[click in window to advance]").SetPosition(new Point(0, 0, 0)).SetAlignment(Text.Alignment.Center);
            SegmentKey textSegmentKey = modelKey.Down("");
            textSegmentKey.GetDrawingAttributeControl().SetDepthRange(0, 0); // bring to front
            TextKey textKey = textSegmentKey.InsertText(textKit);


            canvas.Update();
            myWindowKey.Pause();

            textKey.SetText("Styled with opaque red style\n[click to advance]");

            // set the highlight style, which must be predefined
            highlightOperator.SetHighlightOptions(hok1);

            // inject mouse event to perform selection
            MouseButtons mb = new MouseButtons();
            mb.Left(true);
            MouseEvent mouseEvent = new MouseEvent(HPS.MouseEvent.Action.ButtonDown, new WindowPoint(), mb);
            EventNotifier eventNotifier = myWindowKey.GetEventDispatcher().InjectEventWithNotifier(mouseEvent);
            eventNotifier.Wait();

            canvas.Update();
            myWindowKey.Pause();

            textKey.SetText("Styled with transparent red style\n[click to advance]");
            hok2.SetOverlay(HPS.Drawing.Overlay.InPlace); // overlay "InPlace" is necessary for transparent styles

            // set the highlight style, which must be predefined
            highlightOperator.SetHighlightOptions(hok2);

            // inject mouse event to perform selection
            eventNotifier = myWindowKey.GetEventDispatcher().InjectEventWithNotifier(mouseEvent);
            eventNotifier.Wait();

            canvas.Update();
            myWindowKey.Pause();

            textKey.SetText("Styled with textured style\n[click to advance]");
            hok2.SetOverlay(HPS.Drawing.Overlay.OverlayWithZValues);

            // set the highlight style, which must be predefined
            highlightOperator.SetHighlightOptions(hok3);

            // inject mouse event to perform selection
            eventNotifier = myWindowKey.GetEventDispatcher().InjectEventWithNotifier(mouseEvent);
            eventNotifier.Wait();

            canvas.Update();
        }
    }
}
