using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Image_Io(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            /* Sample: Image_Io
	         * This sample will show you how to:
	         * - Load a JPG file through HPS::Image and check for Image IO exceptions
	         * - Obtain an ImageKit
	         * - Convert an image between formats
	         * - Output an image to a file
	         * - Insert text
	         * 
	         * Read more about the topics covered in this sample in our Programming Guide 
	         * - section 8.3:	Loading 2D Images */

            /* This next block of code shows you how to load an HSF file
             * 1. Create an ImportOptionsKit and set the format of the image to be read in
             * 2. Use the Image::File::Import method to load the desired image, with the options you specified
             * 3. An ImageKit describing the inserted image is returned by Image::File::Import
             * 4. Use a try-catch block to capture any Image IO exceptions */

            ImageKit jpgImageKit = new ImageKit();
	        uint imageWidth, imageHeight;
	        try
	        {
                string file_in = INPUT_FOLDER + "/sprocket_image.jpg";
		        Image.ImportOptionsKit importOptionsKit = new Image.ImportOptionsKit();
		        importOptionsKit.SetFormat(Image.Format.Jpeg);
		        jpgImageKit = Image.File.Import(file_in, importOptionsKit);
	        }
	        catch (IOException)
	        {
		        string error_message;
		        error_message = "Image Import Unsuccessful";
		        modelKey.InsertText(new Point(0, 0, 0), error_message);
		        view.FitWorld();
		        view.Update();
	        }

            /* The next block of code shows how to convert between image formats and how to write an image to a file
	         * 1. Create a new ImageKit object and use its Convert method to change formats
	         * 2. The first argument of Convert is the source ImageKit
	         * 3. The second argument of Convert is the desired output format
	         * 4. Once the new ImageKit is obtained through Convert, obtain its data using the ImageKit::ShowData() method
	         * 5. Write the data out to a file in binary mode */
	        if(!jpgImageKit.Empty())
	        {
		        bool exportSuccessful = true;
		        ImageKit pngImageKit = new ImageKit();
		        byte[] imageData;

		        jpgImageKit.ShowSize(out imageWidth, out imageHeight);
		        pngImageKit.SetFormat(Image.Format.Png).SetSize(imageWidth, imageHeight);
		        pngImageKit.Convert(jpgImageKit, Image.Format.Png);
		        pngImageKit.ShowData(out imageData);
		
		        string file_out = OUTPUT_FOLDER + "/image_io.png";

                try
                {
                    using(System.IO.BinaryWriter writer = new System.IO.BinaryWriter(System.IO.File.OpenWrite(file_out)))
                    {
                        writer.Write(imageData);
                        writer.Close();
                    }
                }
                catch(System.Exception)
                {
                    exportSuccessful = false;
                }

		        if(!exportSuccessful)
		        {
			        string error_message;
			        error_message = "Image Export Unsuccessful";
			        modelKey.InsertText(new Point(0, 0, 0), error_message);
			        view.FitWorld();
			        view.Update();
		        }
		        else
		        {
                    /* The next block of code shows you how to delete geometry from a segment and how to insert text
			         * 1. The Flush method can be used to delete different elements from a segment
			         * 2. The first argument of Flush specifies what to delete
			         * 3. The second argument specifies which part of the scene graph will be affected by this operation
			         * 4. To insert text, specify the text position and the text you want displayed*/
                    string success_message = "Import and Export completed successfully\n" +
				        "You can find the PNG image here:\n" + 
				        OUTPUT_FOLDER + "/image_io.png";

			        view.GetAttachedModel().GetSegmentKey().Flush(Search.Type.Everything, Search.Space.SubsegmentsAndIncludes);
			        view.GetAttachedModel().GetSegmentKey().InsertText(new Point(0, 0, 0), success_message);
			        view.FitWorld();
			        view.Update();
		        }
		
	        }
        }
    }
}
