using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Publish_Widgets(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            
            try
            {
                /* Sample: Publish_Widgets
                * This sample will show you how to:
                * - Create a PDF file with widgets using HOOPS Publish
                * - Output a the file through Publish and check for IO exceptions
                *
                * Read more about the topics covered in this sample in our Programming Guide
                * - Integration with HOOPS Publish */
                HPS.Publish.PageKit pageKit = new Publish.PageKit();             // corresponds to a page inside the PDF
                HPS.Publish.DocumentKit documentKit = new Publish.DocumentKit(); // corresponds to the document itself as a whole

                // the text that will exist next to the checkbox
                Publish.TextKit text = new Publish.TextKit();
                text.SetText("checkbox");
                pageKit.AddText(text, new HPS.IntRectangle(140, 260, 400, 450));

                // setting up the checkbox
                Publish.CheckBoxKit checkBoxKit = Publish.CheckBoxKit.GetDefault();
                checkBoxKit.SetVisibility(true).
                    SetTextColor(new RGBColor(0, 0, 0)).
                    SetFillColor(new RGBColor(0.5f, 0.5f, 0.5f)).
                    SetName("cb1").
                    SetDefaultState(true);

                // checkboxes can be any size, as specified by the IntRectangle. this one will be fairly large
                pageKit.AddCheckBox(checkBoxKit, new HPS.IntRectangle(10, 120, 400, 450));

                // add a list button to the page
                Publish.ListBoxKit list_box_kit = new Publish.ListBoxKit();
                String[] displayed_contents = new String[] { "value1", "value2", "value3" };
                String[] exported_contents = new String[] { "e1", "e2", "e3" };

                list_box_kit.SetName("lb2").
                    SetContents(displayed_contents, exported_contents);
                pageKit.AddListBox(list_box_kit, new HPS.IntRectangle(10, 60, 300, 350));

                // add button to the page
                Publish.ButtonKit buttonKit = new Publish.ButtonKit();
                buttonKit.SetLabel("This is a page button")
                    .SetLabelPosition(Publish.Label.Position.OnTop)
                    .SetHighlighting(Publish.Highlighting.Mode.Push)
                    .SetVisibility(true)
                    .SetFont(Publish.Text.Font.Name.TimesRoman)
                    .SetFontSize(8)
                    .SetTextColor(new RGBColor(0, 0, 0))
                    .SetFillColor(new RGBColor(0.5f, 0.5f, 0.5f))
                    .SetName("execute_button"); // setting the name of the button is required!
                pageKit.AddButton(buttonKit, new HPS.IntRectangle(10, 120, 500, 550));

                // will print "hello world" to the Javascript console
                pageKit.SetJavaScriptActionByField("execute_button", "console.println(\"hello world\");", HPS.Publish.Source.Type.Code);

                documentKit.AddPage(pageKit);

                HPS.Publish.File.Export(documentKit, OUTPUT_FOLDER + "publish_widgets.pdf", new Publish.ExportOptionsKit());
            }
            catch (IOException e)
            {
                System.Console.WriteLine(e.Message);
            }
            
			   
            
        }
    }
}
