using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Sub_Shell_Attributes(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {

            /* Sample: Sub_Shell_Attributes
	         * This sample will show you how to:
	         * - Insert a shell
	         * - Divide the screen into sub-window
	         * - Change shell level attributes
	         * 
	         * Read more about the topics covered in this sample in our Programming Guide 
	         * - section 2.1:	Shells */

            SubwindowKit subwindowKit = new SubwindowKit();
	        VisibilityKit visibilityKit = new VisibilityKit();

            modelKey.GetTextAttributeControl().SetAlignment(Text.Alignment.Center);

            /* This block of code sets visibility and color attributes at the segment level
        	 * Shell level attributes take precedence over segment level attributes */
	        visibilityKit.SetFaces(true).SetEdges(true).SetVertices(true);
	        modelKey.SetVisibility(visibilityKit);
	        modelKey.GetMaterialMappingControl().SetFaceColor(new RGBAColor(1, 0, 0, 1)).SetEdgeColor(new RGBAColor(0, 0, 1, 1)).
		        SetVertexColor(new RGBAColor(0, 1, 0, 1));

            /* This block of code divides the screen into sub-windows. 
	         * Each segment gets its own sub-window, all segments are direct children of the modelKey segment
	         * 1. To set a sub-window call the SetSubwindow method with the dimensions of the sub-window, 
	         *		normalized between -1 and 1, with (-1, 1) pointing to the top left corner */

	        //Vertex Visibility
	        SegmentKey box = modelKey.Subsegment("box");
	        subwindowKit.SetSubwindow(new HPS.Rectangle(-1, 0, 0, 1));
	        ShellKey sk1 = CreateBox(box, new Point(-1, -1, -1), new Point(1, 1, 1));
	        sk1.SetVertexVisibilitiesByRange(0, 8, false);
	        box.SetSubwindow(subwindowKit);
	        box.InsertText(new Point(0, 1.25f, 0), "Hidden Markers");

	        //Hidden Faces
	        SegmentKey hiddenBox = modelKey.Subsegment("hidden_box");
	        subwindowKit.SetSubwindow(new HPS.Rectangle(0, 1, 0, 1));
	        ShellKey sk2 = CreateBox(hiddenBox, new Point(-1, -1, -1), new Point(1, 1, 1));
	        sk2.SetFaceVisibilitiesByRange(0, 6, false);
	        hiddenBox.SetSubwindow(subwindowKit);
	        hiddenBox.InsertText(new Point(0, 1.25f, 0), "Hidden Faces");

	        //Change of Color
	        SegmentKey yellowBox = modelKey.Subsegment("yellow_box");
	        subwindowKit.SetSubwindow(new HPS.Rectangle(-1, 0, -1, 0));
	        ShellKey sk3 = CreateBox(yellowBox, new Point(-1, -1, -1), new Point(1, 1, 1));
	        sk3.SetFaceRGBColorsByRange(0, 6, new RGBColor(1, 1, 0));
	        yellowBox.SetSubwindow(subwindowKit);
	        yellowBox.InsertText(new Point(0, 1.25f, 0), "A Yellow Box");

	        //Deleted Box
	        SegmentKey deletedBox = modelKey.Subsegment("deleted_box");
	        subwindowKit.SetSubwindow(new HPS.Rectangle(0, 1, -1, 0));
	        ShellKey sk4 = CreateBox(deletedBox, new Point(-1, -1, -1), new Point(1, 1, 1));
	        sk4.Delete();
	        deletedBox.SetSubwindow(subwindowKit);
	        deletedBox.InsertText(new Point(0, 1.25f, 0), "The Box is Gone!");

	        view.FitWorld();
	        view.GetSegmentKey().GetCameraControl().Orbit(35, 25);


        }
    }
}
