#if defined __APPLE__
#include "SamplesOSX.h"
#else
#include "sample.h"
#endif
#include "sample_common.h"
#include "sprk.h"

#if defined(_MSC_VER)
#include "sprk_exchange.h"
#include "sprk_publish.h"
#endif

#include <iostream>

using namespace HPS;

void Sample::Exchange_Publish(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
{
#if defined(_MSC_VER)
	try
	{

	/* Sample: Exchange_Publish
	 * This sample will show you how to:
	 * - Load an Exchange file and check for IO exceptions
	 * - Output a 3D PDF file through Publish and check for IO exceptions
	 * 
	 * Read more about the topics covered in this sample in our Programming Guide 
	 * - Integration with HOOPS Exchange
	 * - Integration with HOOPS Publish */

	/* This next block of code shows you how to load an Exchange file
	 * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
	 * 2. Use the Exchange::File::Import method to load the desired file, with the options you specified
	 * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
	 * 4. Use a try-catch block to capture any Stream IO exceptions 
	 * 5. Obtain an Exchange::CADModel object through the Exchange::ImportNotifier
	 * 6. Obtain the new View from the CADModel object. Attach the new view to the Canvas and delete the old View
	 *		You will need to re-insert geometry that was in the old View, such as lights.
	 *		You will also need to re-apply any operators which were active on the old view.*/
	SegmentKey textSeg = wk.Down("text", true);
	textSeg.GetTextAttributeControl().SetAlignment(Text::Alignment::Center);
	UTF8 message;

	Exchange::ImportNotifier exchangeNotifier;
	try {
		UTF8 file_in = INPUT_FOLDER + "/Crank-Instructions.prc";
		exchangeNotifier = Exchange::File::Import(static_cast<const char*>(file_in), Exchange::ImportOptionsKit());
		exchangeNotifier.Wait();
	}

	catch(IOException ioe) {
		message = "Exchange Import Unsuccessful";
		textSeg.InsertText(Point(0, 0, 0), static_cast<const char*>(message));
		view.FitWorld();
		view.Update();
	}

	CADModel modelFile = exchangeNotifier.GetCADModel();
	view.Delete();
	view = modelFile.ActivateDefaultCapture();
	canvas.AttachViewAsLayout(view);
	modelKey.InsertDistantLight(Vector(1, 1, 0));
	view.FitWorld();

	/* This next block of code shows you how to output a 3D PDF file
	 * 1. Create SprocketPath object, by passing the keys to the canvas, layout, view and model to it
	 * 2. Use the Publish::File::Export method to output a 3D PDF file */
	if(exchangeNotifier.Status() == IOResult::Success)
	{
		bool publishExportSuccess = true;
		UTF8 file_out = OUTPUT_FOLDER + "/exchange_publish_sample.pdf";

		try {
			SprocketPath sprkPath(canvas, canvas.GetAttachedLayout(), 
				canvas.GetAttachedLayout().GetAttachedView(),
				canvas.GetAttachedLayout().GetAttachedView().GetAttachedModel());
			Publish::File::Export(sprkPath, static_cast<const char*>(file_out), Publish::ExportOptionsKit());
		}

		catch(IOException ioe) {
			message = "Publish Export Unsuccessful";
			publishExportSuccess = false;
			textSeg.InsertText(Point(0, 0, 0), static_cast<const char*>(message));
			view.FitWorld();
			view.Update();
		}

		if(publishExportSuccess)
		{
			message =
				"Import and Export completed successfully\n"
				"You can find the 3D PDF here:\n" +
				file_out;

			view.GetAttachedModel().GetSegmentKey().Flush(Search::Type::Everything, Search::Space::SubsegmentsAndIncludes);
			textSeg.InsertText(Point(0, 0, 0), static_cast<const char*>(message));
			view.FitWorld();
			view.Update();
		}

	}

	}
	catch(IOException e)
	{
		std::cout << e.what() << std::endl;
	}
#else
	model.GetSegmentKey().InsertText(Point(0, 0, 0), "HOOPS Publish is not supported on this platform");
	model.GetSegmentKey().GetVisibilityControl().SetText(true);
	model.GetSegmentKey().GetTextAttributeControl().SetAlignment(HPS::Text::Alignment::Center);
#endif
}
