using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public List<Test> availableTests;
        public Test testToRun;
        public string driver;
        public string INPUT_FOLDER;
        public string OUTPUT_FOLDER;

        public Sample(World world)
        {
            availableTests = new List<Test>();
            availableTests.Add(new Test(0,  "lightweight_subwindow"));
            availableTests.Add(new Test(1,  "bloom_silhouette"));
            availableTests.Add(new Test(2,  "database_search"));
            availableTests.Add(new Test(3,  "hidden_line"));
            availableTests.Add(new Test(4,  "image_background"));
            availableTests.Add(new Test(5,  "image_io"));
            availableTests.Add(new Test(6,  "image_output"));
            availableTests.Add(new Test(7,  "mask_transforms"));
            availableTests.Add(new Test(8,  "meshes"));
            availableTests.Add(new Test(9,  "quickmoves"));
            availableTests.Add(new Test(10, "render_modes"));
            availableTests.Add(new Test(11, "segment_attributes"));
            availableTests.Add(new Test(12, "sub_shell_attributes"));
            availableTests.Add(new Test(13, "spotlight"));
            availableTests.Add(new Test(14, "textures"));
            availableTests.Add(new Test(15, "exchange_publish"));
            availableTests.Add(new Test(16, "hidden_line_advanced"));
            availableTests.Add(new Test(17, "shell_attributes"));
            availableTests.Add(new Test(18, "priority"));
            availableTests.Add(new Test(19, "interpolation"));
            availableTests.Add(new Test(20, "exchange_pmi_highlighting"));
            availableTests.Add(new Test(21, "exchange_component_highlighting"));
            availableTests.Add(new Test(22, "custom_event"));
            availableTests.Add(new Test(23, "conditional_styles_and_includes"));
            availableTests.Add(new Test(24, "hardcopy_export"));
            availableTests.Add(new Test(25, "selection"));
            availableTests.Add(new Test(26, "exchange_component_interaction"));
            availableTests.Add(new Test(27, "insert_cube"));
            availableTests.Add(new Test(28, "select_by_point"));
            availableTests.Add(new Test(29, "highlighting"));
            availableTests.Add(new Test(30, "publish_widgets"));
            availableTests.Add(new Test(31, "complex_clip_regions"));

            driver = "OpenGL2";
            testToRun = new Test();

			string executableDirectory = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
            INPUT_FOLDER = executableDirectory + "/../../samples/data";
            OUTPUT_FOLDER = executableDirectory + "/../../samples/code/output";

            InitHPS(world);
        }

        private void InitHPS(World world)
        {
			string executableDirectory = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
            world.SetMaterialLibraryDirectory(executableDirectory + "/../../samples/data/materials");
		    string HEXCHANGE_INSTALL_DIR = Environment.GetEnvironmentVariable("HEXCHANGE_INSTALL_DIR");
            if (HEXCHANGE_INSTALL_DIR != null)
            {
                string bin_subdir;
#               if WIN64
		            bin_subdir = "win64/";
#               else
                    bin_subdir = "win32/";
#               endif
                world.SetExchangeLibraryDirectory(HEXCHANGE_INSTALL_DIR + "/bin/" + bin_subdir);
                world.SetPublishResourceDirectory(HEXCHANGE_INSTALL_DIR + "/bin/resource");
            }
        }//InitHPS

        public bool ProcessArguments(string[] args)
        {
            if(args.Length < 2)
                return false;
            for (int i = 0; i < args.Length; i++)
            {
                try
                {
                    if (args[i][0] != '-')
                        return false;

                    if (args[i][1] == 't')
                    {
                        testToRun.testName = args[i + 1];
                        Test item = availableTests.Find(
                            delegate(Test t)
                            {
                                return t.testName == testToRun.testName;
                            }
                            );
                        if (item != null)
                            testToRun.testID = item.testID;
                        else
                            return false;
                    }
                    else if (args[i][1] == 'd')
                    {
                        try { driver = args[i + 1]; }
                        catch (System.IndexOutOfRangeException)
                        { driver = "opengl2"; }
                    }  
                    else
                        return false;
                }
                catch (System.IndexOutOfRangeException)
                {
                    return false;
                }

                i++;
            }

            return true;
        }

        public void RunTest(Test test, string driver)
        {
            Window.Driver driverToUse;

			if (driver.ToLower() == "directx11" || driver.ToLower() == "dx11")
				driverToUse = Window.Driver.DirectX11;
			else if (driver.ToLower() == "directx9" || driver.ToLower() == "dx9")
		        driverToUse = Window.Driver.DirectX9;
            else if (driver.ToLower() == "opengl")
                driverToUse = Window.Driver.OpenGL;
            else
            {
                driverToUse = Window.Driver.OpenGL2;
                driver = "opengl2";
            }

            StandAloneWindowOptionsKit wo =  new StandAloneWindowOptionsKit();
            wo.SetMobility(Window.Mobility.Free);
	        wo.SetSubscreen(new HPS.Rectangle(-0.75f, 0.75f, -0.75f, 0.75f));
	        wo.SetDriver(driverToUse);
	        string windowName = "HPS Samples - " + testToRun.testName + " - " + driver;
	        wo.SetTitle(windowName);

            StandAloneWindowKey wk = Database.CreateStandAloneWindow(wo);
	        Canvas canvas = Factory.CreateCanvas(wk);
	        View view = Factory.CreateView();
            canvas.AttachViewAsLayout(view);
	        Model model = Factory.CreateModel();
	        view.AttachModel(model);
	        SegmentKey modelKey = model.GetSegmentKey();
	        modelKey.InsertDistantLight(new Vector(1, 1, 0));

            switch (test.testID)
            {
                case 0:
                    Lightweight_Subwindow(wk, canvas, view, model, modelKey);
                    break;
                case 1:
                    Bloom_Silhouette(wk, canvas, view, model, modelKey);
                    break;
                case 2:
                    Database_Search(wk, canvas, view, model, modelKey);
                    break;
                case 3:
                    Hidden_Line(wk, canvas, view, model, modelKey);
                    break;
                case 4:
                    Image_Background(wk, canvas, view, model, modelKey);
                    break;
                case 5:
                    Image_Io(wk, canvas, view, model, modelKey);
                    break;
                case 6:
                    Image_Output(wk, canvas, view, model, modelKey);
                    break;
                case 7:
                    Mask_Transforms(wk, canvas, view, model, modelKey);
                    break;
                case 8:
                    Meshes(wk, canvas, view, model, modelKey);
                    break;
                case 9:
                    Quickmoves(wk, canvas, view, model, modelKey);
                    break;
                case 10:
                    Render_Modes(wk, canvas, view, model, modelKey);
                    break;
                case 11:
                    Segment_Attributes(wk, canvas, view, model, modelKey);
                    break;
                case 12:
                    Sub_Shell_Attributes(wk, canvas, view, model, modelKey);
                    break;
                case 13:
                    Spotlight(wk, canvas, view, model, modelKey);
                    break;
                case 14:
                    Textures(wk, canvas, view, model, modelKey);
                    break;
                case 15:
                    Exchange_Publish(wk, canvas, ref view, model, modelKey);
                    break;
                case 16:
                    Hidden_Line_Advanced(wk, canvas, view, model, modelKey);
                    break;
                case 17:
                    Shell_Attributes(wk, canvas, view, model, modelKey);
                    break;
                case 18:
                    Priority(wk, canvas, view, model, modelKey);
                    break;
                case 19:
                    Interpolation(wk, canvas, view, model, modelKey);
                    break;
                case 20:
                    Exchange_PMI_Highlighting(wk, canvas, ref view, model, modelKey);
                    break;
                case 21:
                    Exchange_Component_Highlighting(wk, canvas, ref view, model, modelKey);
                    break;
                case 22:
                    Custom_Event(wk, canvas, view, model, modelKey);
                    break;
                case 23:
                    Conditional_Styles_And_Includes(wk, canvas, view, model, modelKey);
                    break;
                case 24:
                    Hardcopy_Export(wk, canvas, view, model, modelKey);
                    break;
                case 25:
                    Selection(wk, canvas, view, model, modelKey);
                    break;
                case 26:
                    Exchange_Component_Interaction(wk, canvas, ref view, model, modelKey);
                    break;
                case 27:
                    Insert_Cube(wk, canvas, view, model, modelKey);
                    break;
                case 28:
                    Select_By_Point(wk, canvas, view, model, modelKey);
                    break;
                case 29:
                    Highlighting(wk, canvas, view, model, modelKey);
                    break;
                case 30:
                    Publish_Widgets(wk, canvas, view, model, modelKey);
                    break;
                case 31:
                    Complex_Clip_Regions(wk, canvas, view, model, modelKey);
                    break;
            }

			wk.UpdateWithNotifier().Wait();
            wk.Pause();
        }//RunTest
    }

    public class Test
    {
        public Test()
        {
            testID = -1;
            testName = "";
        }

        public Test(int ID, string name)
        {
            testID = ID;
            testName = name;
        }

        public int testID;
        public string testName;
    }
}
