using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public class CustomEvent : Event
        {
			public CustomEvent() { }
			public CustomEvent(IntPtr type) : base(type) { }
			public CustomEvent(IntPtr type, SegmentKey sKey)
                : base(type)
            {
                textSegmentKey = sKey;
            }

            public override Event Clone()
            {
                CustomEvent ce = new CustomEvent(GetChannel());
                ce.textSegmentKey = this.textSegmentKey;
                return ce;
            }

            public SegmentKey textSegmentKey;
        };

        public class CustomEventHandler : HPS.EventHandler
        {
            public override HPS.EventHandler.HandleResult Handle(Event in_event)
            {
                SegmentKey textKey = ((CustomEvent)in_event).textSegmentKey;
                SegmentKey labelKey = textKey.Subsegment();
                labelKey.GetDrawingAttributeControl().SetDepthRange(0.0f, 0.1f);
                labelKey.GetVisibilityControl().SetEverything(false).SetText(true);
                labelKey.GetCameraControl().SetPosition(new Point(0, 0, -5)).
                    SetTarget(new Point(0, 0, 0)).
                    SetUpVector(new Vector(0, 1, 0)).
                    SetField(2, 2).
                    SetProjection(Camera.Projection.Stretched);
                labelKey.GetTextAttributeControl().SetSize(0.02f, Text.SizeUnits.SubscreenRelative).
                    SetAlignment(Text.Alignment.Center, Text.ReferenceFrame.WorldAligned, Text.Justification.CenterJustified);
                labelKey.InsertText(new Point(0, 0, 0), "Your custom event\n has been handled successfully");

                return HPS.EventHandler.HandleResult.Handled;
            }
        };

        private float DegreesToRadians(float degrees)
        {
            return (float)(degrees * Math.PI / 180.0f);
        }

        private MeshKey CreateToroid(SegmentKey insertHere)
        {
            float golden;
            float cx, cy, cz;
            float cosa, sina;
            Vector basis1, basis2;

            List<Point> points = new List<Point>();
            List<Vector> normals = new List<Vector>();

            golden = (float)(Math.Sqrt(5) + 1.0f) * 0.5f;
            cz = 0.0f;

            for (int i = 0; i <= 360; i += 10)
            {
                cx = golden * (float)Math.Cos(DegreesToRadians((float)i));
                cy = (float)Math.Sin(DegreesToRadians((float)i));

                basis1.x = 0.0f;
                basis1.y = 0.0f;
                basis1.z = 1.0f;

                basis2.y = golden * (float)Math.Sin(DegreesToRadians((float)i));
                basis2.x = (float)Math.Cos(DegreesToRadians((float)i));
                basis2.z = 0.0f;
                basis2.Normalize();

                for (int j = 0; j <= 360; j += 15)
                {
                    Point point;
                    Vector normal;
                    cosa = (float)Math.Cos(DegreesToRadians((float)j));
                    sina = (float)Math.Sin(DegreesToRadians((float)j));

                    point.x = cx + 0.2f * (cosa * basis1.x + sina * basis2.x);
                    point.y = cy + 0.2f * (cosa * basis1.y + sina * basis2.y);
                    point.z = cz + 0.2f * (cosa * basis1.z + sina * basis2.z);
                    points.Add(point);

                    normal.x = cosa * basis1.x + sina * basis2.x;
                    normal.y = cosa * basis1.y + sina * basis2.y;
                    normal.z = cosa * basis1.z + sina * basis2.z;
                    normals.Add(normal);
                }//end of inner for loop
            }//end of outer for loop

            MeshKey ret = insertHere.InsertMesh((ulong)37, (ulong)25, points.ToArray());

            for (int i = 0; i <= 360; i += 10)
            {
                for (int j = 0; j <= 360; j += 15)
                {
                    ret.SetVertexNormalsByRange(0, normals.ToArray());
                }
            }

            return ret;
        }

        private SegmentKey InsertCross(SegmentKey insertHere)
        {
            SegmentKey crossSegment = insertHere.Subsegment();
            crossSegment.InsertLine(new Point(0, 0, 0), new Point(1, 0, 0));
            crossSegment.InsertLine(new Point(0.5f, 0.5f, 0), new Point(0.5f, -0.5f, 0));
            crossSegment.GetVisibilityControl().SetLines(true);
            crossSegment.GetLineAttributeControl().SetWeight(3);

            return crossSegment;
        }

        private ShellKey CreateBox(SegmentKey insertHere, Point min, Point max)
        {
            Point[] points = new Point[8];
            Point point = new Point();

            point.x = min.x;
            point.y = min.y;
            point.z = min.z;
            points[0] = point;

            point.x = max.x;
            points[1] = point;

            point.y = max.y;
            points[2] = point;

            point.x = min.x;
            points[3] = point;

            point.y = min.y;
            point.z = max.z;
            points[4] = point;

            point.x = max.x;
            points[5] = point;

            point.y = max.y;
            points[6] = point;

            point.x = min.x;
            points[7] = point;

            int[] faces = {4, 0, 1, 2, 3, 4, 7, 6, 5, 4, 
			                4, 1, 5, 6, 2, 4, 4, 0, 3, 7, 
			                4, 3, 2, 6, 7, 4, 4, 5, 1, 0};

            return insertHere.InsertShell(points, faces);
        }
    }
}
